@@ -40,7 +40,7 @@ def clean_generated_resources(path: str = 'src/main/resources'):
             # File, check if valid and then delete
             if subdir.endswith('.json'):
                 delete = False
-                with open(sub_path, 'r') as file:
+                with open(sub_path, 'r', encoding='utf-8') as file:
                     if '"__comment__": "This file was automatically created by mcresources"' in file.read():
                         delete = True
                 if delete:
@@ -80,12 +80,12 @@ def write(path_parts: Sequence[str], data: Json, indent: int = 2, on_error: Call
         os.makedirs(os.path.dirname(path), exist_ok=True)
         exists = False
         if os.path.isfile(path):
-            with open(path, 'r') as file:
+            with open(path, 'r', encoding='utf-8') as file:
                 old_data = json.load(file)
             if old_data == data:
                 return WriteFlag.UNCHANGED
             exists = True
-        with open(path, 'w') as file:
+        with open(path, 'w', encoding='utf-8') as file:
             json.dump(data, file, indent=indent)
             return WriteFlag.MODIFIED if exists else WriteFlag.NEW
     except Exception as e:
@@ -238,15 +238,12 @@ def item_stack_dict(data_in: Union[Dict[str, Any], str], default_char: str = '#'
         raise RuntimeError('Unknown object %s at item_stack_dict' % str(data_in))
 
 
-def item_model_textures(data_in: Json) -> Dict[str, Any]:
-    if isinstance(data_in, str):
-        return {'layer0': data_in}
-    elif isinstance(data_in, Sequence):
-        return dict(('layer%d' % i, data_in[i]) for i in range(len(data_in)))
-    elif isinstance(data_in, Dict):
-        return data_in
+def item_model_textures(data_in: Tuple[Json, ...]) -> Dict[str, Any]:
+    # Input must be in tuple (varargs) format
+    if len(data_in) == 1 and isinstance(data_in[0], Dict):
+        return data_in[0]
     else:
-        raise RuntimeError('Unknown object %s at item_model_textures' % str(data_in))
+        return dict(('layer%d' % i, layer) for i, layer in enumerate(flatten_list(data_in)))
 
 
 def blockstate_multipart_parts(data_in: Sequence[Json]) -> List[Dict[str, Any]]:
